// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_VDW_H_
#define _SX_VDW_H_

#include <SxDFT.h>
#include <SxAtomicStructure.h>
#include <SxSpeciesData.h>
#include <SxSymbolTable.h>

/** This class is for the empirical van der waals correction
    \author Lars Ismer
    \author Michael Ashton, ashton@mpie.de
 */
class SX_EXPORT_DFT SxVDW
{
   public:
	 
	  SxString correctionType;
      SxString combinationRule;
      SxString damping;
	  SxArray<SxVector3<Double> > coord;
	  SxArray<SxString> species;
	  
	  SxMatrix3<Double> superCell;
	  SxArray<double>  energyContrib;

	  SxArray<SxList<int> > neighbours;
      SxArray<SxList<int> > sNeighbours;

	  SxArray<SxList<int> >  borderType;
	  SxArray<SxList<int> > supercellIds;
	  SxArray<SxList<int> > sSupercellIds;
	  
      SxArray<SxList<int> > borderCrossers;
	  SxArray<SxVector3<Double> > Forces;

	  SxArray<SxList<double> > dist;
	  SxArray<SxList<double> > expTerm;

	  SxArray<double> polarizability;
	  SxArray<double> C6;
	  SxArray<double> vdwRadius;


	  bool output;
	  int nAtoms;
	  double totalEnergy;

	   SxVDW ();
      SxVDW (const SxAtomicStructure &, const SxSymbolTable *);
     ~SxVDW ();

	  
	  double getDist(int, int, int);
	  void resize 
		  (SxList<SxList<SxVector3<Double> > > , SxList<SxString>, SxMatrix3<Double>);
	  void update(SxArray<SxVector3<Double> >);
	  void updateNeighbours (SxString);

	  void updateCoord (SxArray<SxVector3<Double> >);
	  void updateBorderCrossers ();
	  
	  void updateDampingTerms (); 
	  void updateHybridisation (); 
	  
	  void compute ();

	  SxVector3<Double>  getLatticeVec(int);
	  SxVector3<Double>  getForceOnAtom (int); 
	  void updateForces (); 
	  SxArray<SxVector3<Double> > getForces ();
	  SxArray<SxVector3<Double> > getNumericalForces (double);

	  bool areNeighbors(int,int);
	  int getNeighborIndex(int,int);
	  double getDampingFunction(double, double);
	  double getDampingDerivative(double, double);
	  double getDampingSecondDerivative(double, double);
	  
      double getVolumeFraction(int);
	  double getC6ij (int, int);
	  double getRij (int, int);
	  double getParam (SxString, int, int);
	  double getTotalEnergy ();

      int getnAtoms ();

	  void printParameters ();
	  void printNeighbours ();
	  
	  void printCoord ();
	  void printSpecies ();
};


#endif /* _SX_VDW_H_ */
